import sys, argparse,os
from .configManager import load_config,get_variables
from .LambdaFunction import LambdaFunction
from .S3 import S3Bucket
from .Zip import Zip



def deploy(config):
    bucket_name = os.environ["LAMBDA_CODE_BUCKET"]
    code_zip = Zip(name=config["FUNCTION_NAME"],path=config["CODE_DIR"]).name
    function = LambdaFunction(config["FUNCTION_NAME"])
    code_bucket = S3Bucket(bucket_name)
    code_bucket.upload_file(code_zip)
    variables = get_variables(config)
    function.deploy(bucket_name,code_zip)
    function.update_env_variables(variables)
    version = function.publish_version(config["VERSION"])["Version"]
    function.update_alias(alias="stable",description=config["VERSION"],version=version)

def parse_args():
    parser = argparse.ArgumentParser(description='Deploy lambda function')
    parser.add_argument(
        '-c',
        '--config',
        type=str,
        help='config directory or file',
        required=True
    )
    return parser.parse_args()

def main():
    args = parse_args()
    config = load_config(args.config)
    deploy(config)


import os,sys
from dotenv.main import dotenv_values


def load_config(path):
    try:
        return dotenv_values(dotenv_path=path)
    except FileNotFoundError:
        print('Invalid path: {}'.format(path))
        sys.exit(1)


def get_variables(config):
    return {**get_environment_variables(config), **get_secret_variables(config)}
    
def get_environment_variables(config):
    env_keys = list(filter(lambda variable: variable.startswith('ENV_'), config.keys()))
    variables = {}
    for key in env_keys:
        variables[key.replace('ENV_','')] = config[key]
    return variables


def get_secret_variables(config):
    env_keys = list(filter(lambda variable: variable.startswith('SECRET_'), config.keys()))
    variables = {}
    for key in env_keys:
        variables[key.replace('SECRET_','')] = "${"+config[key]+"}"
    return variables



import boto3

class LambdaFunction:

    def __init__(self, name, region='us-east-2'):
        self.name = name
        self.client = boto3.client('lambda',region_name=region)

    def deploy(self, S3bucket,S3key):
        print("Deploying function: {}".format(self.name))
        self.client.update_function_code(
            FunctionName=self.name,
            S3Bucket=S3bucket,
            S3Key=S3key
        ) 

    def update_alias(self,alias,description, version):
        print("Updating alias: {}".format(alias))
        try:
            return self.client.update_alias(
                FunctionName=self.name,
                Name=alias,
                Description=description,
                FunctionVersion=version
            )
        except self.client.exceptions.ResourceNotFoundException:
            print("Alias not exist")
            return self.create_alias(alias,description,version)

    def create_alias(self,alias,description,version):
        print("Creating alias: {}".format(alias))
        return self.client.create_alias(
            FunctionName=self.name,
            Name=alias,
            Description=description,
            FunctionVersion=version
        )

    def publish_version(self,description):
        print("Creating version: {}".format(description))
        return self.client.publish_version(
            FunctionName=self.name,
            Description=description
        )
    
    def update_env_variables(self,variables):
        print("Updating environment variables")
        return self.client.update_function_configuration(
            FunctionName=self.name,
            Environment = {
                "Variables": variables
            }
        )


import os, shutil

class Zip:
    def __init__(self,name,path):
        self.path = self.__compress(name,path)

    def get_bytes(self):
        with open(self.path, 'rb') as file:
            return file.read()

    def __compress(self,name,path):
        print("Compressing code dir: {}".format(path))
        format = "zip"
        self.name = "{}.{}".format(name,format)
        return shutil.make_archive(name, format, path)
        



import boto3
import pathlib
import os
import logging 
from botocore.exceptions import ClientError


class S3Bucket:
    def __init__(self,bucket_name):
        self.bucket_name = bucket_name
        self.client = boto3.client('s3')


    def file_exists(self,file_name):
        try:
            self.client.head_object(Bucket=self.bucket_name,Key=file_name)
        except ClientError as error:
            if error.response["Error"]["Code"] == "404":
                return False
            logging.error(error)
            raise
        return True

    def get_file(self, file_name):
        object = self.client.get_object(Bucket=self.bucket_name,Key=file_name)
        return object["Body"].read().decode('utf-8') 

    def upload_file(self,file_name, object_name=None):
        if not object_name:
            object_name = file_name
        try:
            self.client.upload_file(file_name, self.bucket_name, object_name)
        except ClientError as e:
            logging.error(e)
            return False
        return True

    def upload_folder(self, input_path, destination_path):
        for root, directories, files in os.walk(input_path, topdown=False):
            for name in files:
                input_file_path = os.path.join(root, name)
                destination_file_path = os.path.join(destination_path,name)
                self.upload_file(input_file_path,destination_file_path)
             
    
    def download_file(self, file_name, download_path=None):
        logging.info("Downloading {} from s3".format(file_name))
        if not download_path:
            download_path = file_name
        download_directory = os.path.dirname(download_path)
        pathlib.Path(download_directory).mkdir(parents=True, exist_ok=True)
        self.client.download_file(self.bucket_name,file_name,download_path)


from setuptools import setup,find_packages, os

setup(
    name='deploy-lambda-functions',
    version= os.environ['CIRCLE_TAG'],
    description='Script to deploy lambda functions to AWS',
    url='https://github.com/redaptiveinc/deploy-lambda-functions',
    author='Mariano Gimenez',
    author_email='mariano.gimenez@agileengine.com',
    license='unlicense',
    zip_safe=False,
    packages = find_packages(),
    entry_points ={
        'console_scripts': [
            'deploy-lambda-functions = src.main:main'
        ]
    },
    install_requires = [
        'python-dotenv==0.10.3',
        'boto3==1.10.7',
        'botocore==1.13.9'

    ]
)
